from feedgen.feed import FeedGenerator
from flask import url_for

from mastodon_toot_follower import path_util
from mastodon_toot_follower.conversation.reading_position import ReadingPosition
from mastodon_toot_follower.mastodon_factory import MastodonFactory
from mastodon_toot_follower.conversation.update import Update as ConversationUpdate


class Conversation:
    def __init__(self, mastodon_factory: MastodonFactory, mastodon_instance: str, toot_id: str):
        self.mastodon = mastodon_factory.get_mastodon(mastodon_instance)

        self.toot_id = toot_id
        self.toot = self.mastodon.status(id=self.toot_id)

        escaped_uri = path_util.escape(self.toot["uri"])
        self.file = mastodon_factory.toot_cache_dir().joinpath(f'{escaped_uri}.json')

        # List of replies to initial toot.
        self.replies = self.mastodon.status_context(id=self.toot_id)['descendants']

        previous_updates: list[ConversationUpdate] = ConversationUpdate.load(file=self.file)
        new_updates = ConversationUpdate.get_new_updates(previous_updates, self.toots())
        self._updates = previous_updates + new_updates
        if len(new_updates) > 0:
            ConversationUpdate.save(updates=self._updates, file=self.file)

    def get_and_update_reading_position(self, user_id: str):
        filename = self.file.stem + '_ReadingPosition.json'
        file = self.file.parent.joinpath(filename)

        rp = ReadingPosition(file=file, user_id=user_id)
        new_reading_position = len(self.toots())
        return rp.get_and_set(pos=new_reading_position)

    def as_feed(self, feed_url) -> FeedGenerator:
        """
        Returns a FeedGenerator object "fg".

        atom_feed = fg.atom_str(pretty=True)  # Get the ATOM feed as string
        rss_feed = fg.rss_str(pretty=True)  # Get the RSS feed as string
        """

        fg = FeedGenerator()
        fg.id(self.toot['uri'])  # Unique feed ID.
        fg.title(f'MastodonConversation{self.toot_id}')
        fg.author({'name': self.toot['account']['acct']})
        fg.link(href=self.toot['uri'], rel='alternate')  # Webpage.
        fg.logo(url_for('static', filename='favicon.png'))
        fg.subtitle('RSS feed of a conversation on Mastodon.')
        fg.link(href=feed_url, rel='self')  # URL of the feed itself.
        fg.language('en')

        update: ConversationUpdate
        for update in self._updates:
            fe = fg.add_entry()
            fe.id(update.dict['uri'])
            acct = update.dict['acct']
            if update.dict['reason'] == 'new':
                fe.title(f'Reply by {acct}')
                fe.published(update.dict['created_at'])
            else:
                fe.title(f'Edit by {acct}')
                fe.published(update.dict['edited_at'])
            fe.link(href=update.dict['uri'])
            fe.content(update.dict['content'])

        return fg

    def conversation_length(self) -> int:
        return len(self.toots())

    def updates(self, reading_position: int = 0) -> list:
        """
        :param: reading_position: If given, returns only conversation updates after the reading position.
        """
        return self._updates[reading_position:]

    def toots(self, reading_position: int = 0) -> list:
        """
        :return: List of initial toot and it's replies.
        """
        return [self.toot] + self.replies
