from __future__ import annotations

import inspect
from pathlib import Path
from typing import Literal
import json


class Update:
    @classmethod
    def save(cls, updates: list[Update], file: Path) -> None:
        with file.open('w') as f:
            dict_list = [update.dict for update in updates]
            f.write(json.dumps(dict_list))

    @classmethod
    def load(cls, file: Path) -> list[Update]:
        if not file.exists():
            return []
        dict_list = json.loads(file.read_text())
        return [cls._from_dict(d) for d in dict_list]

    @classmethod
    def get_new_updates(cls,
                        previous_updates: list[Update],
                        toots: list[dict]) -> list[Update]:
        previous_updates_dict = {pu.dict['id']: pu.dict['edited_at']
                                 for pu in previous_updates}
        new_updates: list[Update] = []
        for toot in toots:
            id_not_in_previous_updates = object()
            old_edited_at = previous_updates_dict.get(toot['id'], id_not_in_previous_updates)
            if old_edited_at is id_not_in_previous_updates:
                new_updates.append(Update.new('new', toot))
            elif str(toot['edited_at']) != old_edited_at:
                new_updates.append(Update.new('edited', toot))
        return new_updates

    @classmethod
    def new(cls, reason: Literal['new', 'edited'], toot: dict) -> Update:
        update = Update()
        update.dict = {
            'reason': reason,
            #
            'created_at': str(toot['created_at']),
            'edited_at': str(toot['edited_at']),
            #
            'id': toot['id'],
            'uri': toot['uri'],
            #
            'acct': toot['account']['acct'],
            'username': toot['account']['username'],
            'display_name': toot['account']['display_name'],
            #
            'content': toot['content'],
            'language': toot['language'],
            'media_attachments': toot['media_attachments'],
        }
        return update

    @classmethod
    def _from_dict(cls, d: dict) -> Update:
        update = Update()
        update.dict = d
        return update

    def __init__(self):
        # Make constructor "private":
        # Can only be called from classmethod of Update class.
        stack = inspect.stack()
        if 'cls' not in stack[1][0].f_locals or stack[1][0].f_locals["cls"] is not Update:
            raise ValueError('This constructor is private. Please use one of the classemthods.')

        self.dict = {}

    def get_title(self) -> str:
        acct = self.dict['acct']
        if self.dict['reason'] == 'new':
            return f'Reply by {acct}'
        else:
            return f'Edit by {acct}'

    def get_date(self):
        if self.dict['reason'] == 'new':
            return self.dict['created_at']
        else:
            return self.dict['edited_at']

    def __str__(self):
        return f'{self.get_date()}\n' \
               f'\t{self.get_title()}\n' \
               f'\t{self.dict["uri"]}\n'
